#ifndef _CHANNELMAPEXCEPTION_H
#define _CHANNELMAPEXCEPTION_H

#include <execinfo.h>
#include <stdexcept>
#include <string>
#include <sstream>
#include <cstdlib>

#define GENERATE_CHANNELMAP_EXCEPTION(msg) \
  throw daqchannelmap::ChannelMapException(__FILE__, __LINE__, msg)

namespace daqchannelmap {

/**
 * Class to report exceptions in the DAQChannel Map package.
 *
 * @author A.Norman (based off K.Biery Template)
 * @version $Revision: 1.1 $ $Date: 2014/06/17 19:27:38 $
 */

class ChannelMapException : public std::exception {

  public:

    /**
     * Constructs a data format exception instance from the specified
     * file, line number, and message.
     */
    ChannelMapException(const std::string &fileName, int line,
                        const std::string &message) {

        // convert the line number to a string
        std::stringstream tmpStream;
        tmpStream << line;
        std::string lineNumber = tmpStream.str();

        // build up the exception text
        _fullText = "DAQChannelMap - Error: " + message + " at " + fileName + ":" + lineNumber;

        // add some limited stack trace information
        void *array[10];
        size_t size;
        char **strings;
        size_t idx;

        size    = backtrace (array, 10);
        strings = backtrace_symbols (array, size);
        tmpStream.seekp(0);  // clear out the stream
        for (idx = 0; idx < size; idx++) {
            tmpStream << "      " << strings[idx] << std::endl;
        }
        free (strings);

        _fullText += "\n    Stack trace:\n" + tmpStream.str();
    }

    /**
     * Destructor.
     */
    virtual ~ChannelMapException() throw() { 
        return; 
    }

    /**
     * Returns a description of the exception.
     */
    const char* what() const throw() {
        return _fullText.c_str();
    }

 private:

    /**
     * The full description of the exception.
     */
    std::string _fullText;

};

};  // end of namespace

#endif
