//  This file (RawMilliBlock.h) was created by Ron Rechenmacher <ron@fnal.gov> on
//  Nov  7, 2011. "TERMS AND CONDITIONS" governing this file are in the README
//  or COPYING file. If you do not have such a file, one can be obtained by
//  contacting Ron or Fermi Lab in Batavia IL, 60510, phone: 630-840-3000.
//  $RCSfile: RawMilliBlock.h,v $
//  rev="$Revision: 1.1 $$Date: 2012/09/25 18:39:47 $";

#ifndef RAWMILLIBLOCK_H
#define RAWMILLIBLOCK_H

#include "DAQDataFormats/RawMilliSlice.h"

struct RawMilliBlock
{
    bool     good()              const {return(!(  flags_.timeout_failure
						 ||flags_.buffer_overflow
						 ||flags_.overwrite_failure));}
    bool     timeout_failure()   const {return(flags_.timeout_failure);}
    bool     buffer_overflow()   const {return(flags_.buffer_overflow);}
    bool     overwrite_failure() const {return(flags_.overwrite_failure);}
    uint32_t getSeqNum()         const {return(seqNum_);}
    uint64_t getTimeStart()      const {return(timeStart_);}
    uint64_t getTimeEnd()        const {return(timeEnd_);}

    // 
    int set( uint32_t sn, uint64_t start, uint64_t end, bool ovrflw )
    {   seqNum_=sn; timeStart_=start; timeEnd_=end;
	// could check that end is after begin?
	flags_.buffer_overflow=ovrflw;
	return (0);
    }
    void set_tmo(    bool tmo )    { flags_.timeout_failure   = tmo; }
    void set_ovrwrt( bool ovrwrt ) { flags_.overwrite_failure = ovrwrt; }

    std::vector<daqdataformats::RawMilliSlice> slices;
private:
    uint32_t	seqNum_;
    uint64_t	timeStart_;
    uint64_t	timeEnd_;
    struct
    {   int	timeout_failure:1;
	int	buffer_overflow:1;   //writer side problem.
	int	overwrite_failure:1; //writer overwrote data reader was reading
    }  flags_;
};

#endif // RAWMILLIBLOCK_H
