#ifndef _DBUTILS_DAQCONFIG_DCMRUNCONFIG_H
#define _DBUTILS_DAQCONFIG_DCMRUNCONFIG_H

#include <DatabaseUtils/DAQConfig/GlobalNameSpec.h>
#include <DatabaseUtils/DAQConfig/SubsystemNameSpec.h>
#include <boost/shared_ptr.hpp>
#include <iostream>
#include <string>
#include <vector>

namespace dbutils {
namespace daqconfig {

/**
 * Worker class to handle fetching and formatting of DCM
 * run configuration data.
 *
 * @author Kurt Biery
 * @version $Revision: 1.5 $ $Date: 2014/08/13 22:03:08 $
 */

class DCMRunConfig {

 public:

    // ********************
    // *** Constructors ***
    // ********************

    DCMRunConfig(const std::vector<int64_t>& subsystemConfigIdList,
                 const std::string& dcmName);

    DCMRunConfig(const SubsystemNameSpec& nameSpec,
                 const std::string& dcmName);

    DCMRunConfig(const GlobalNameSpec& nameSpec,
                 const std::string& dcmName);

    DCMRunConfig(const int64_t& globalConfigId,
                 const std::string& dcmName);

    virtual ~DCMRunConfig() {};

    // *****************
    // *** Accessors ***
    // *****************

    virtual bool hasValidData() const {return _hasValidData;}
    int64_t getPrimarySubsystemConfigId() const {return _primarySubsystemConfigId;}
    int32_t getMilliSliceTimeLength() const {return _millisliceTimeLength;}

 protected:

    bool _fetchAllData(const std::vector<int64_t>& subsystemConfigIdList,
                       const std::string& dcmName);

    bool _fetchDCMSystemParams(const std::vector<int64_t>& subsystemConfigIdList);

    bool _fetchDCMDataDevValues(const std::vector<int64_t>& subsystemConfigIdList,
                                const std::string& dcmName);
    bool _fetchDCMApplicationParams(const std::vector<int64_t>& subsystemConfigIdList,
                                    const std::string& dcmName);

    bool _hasValidData;
    int64_t _primarySubsystemConfigId;

    // system-wide parameters
    int32_t _millisliceTimeLength;

    // dcm-specific parameters
    std::string _milliSliceMode;
    int32_t _milliSliceCalculateCheckSum;
    int32_t _corruptMessageLimit;
    int32_t _corruptMessageResetCount;
    int32_t _monitorMetricSendTimeInterval;
    int32_t _monitorCtrlRegGeneralStatus;
    int32_t _numberOfReadAttempts;
    int32_t _maxAllowedDelayInMilliSec;
    int32_t _monitorFEBStatusTimeInterval;
    int32_t _monitorFEBStatusLimit;
};

};  // end of namespace daqconfig
};  // end of namespace dbutil

#endif
