#ifndef _DBUTILS_DAQCONFIG_SUBSYSTEMNAMESPEC_H
#define _DBUTILS_DAQCONFIG_SUBSYSTEMNAMESPEC_H

#include <string>

namespace dbutils {
namespace daqconfig {

/**
 * Simple container class to hold the identifiers that specify a particular
 * named subsystem configuration.
 *
 * @author Kurt Biery
 * @version $Revision: 1.14 $ $Date: 2014/08/01 04:23:30 $
 */

class SubsystemNameSpec {

 public:

    static const std::string DAQMONITOR_SUBSYSTEM_NAME; 
    static const std::string DCM_SUBSYSTEM_NAME;
    static const std::string DL_SUBSYSTEM_NAME;
    static const std::string GT_SUBSYSTEM_NAME;
    static const std::string SM_SUBSYSTEM_NAME;
    static const std::string DDT_SUBSYSTEM_NAME;
    static const std::string TDUMANAGER_SUBSYSTEM_NAME;
    static const std::string BNEVB_SUBSYSTEM_NAME;
    static const std::string CONNECT_CONFIG_TYPE;
    static const std::string HARDWARE_CONFIG_TYPE;
    static const std::string RUN_CONFIG_TYPE;

    // ********************
    // *** Constructors ***
    // ********************

    SubsystemNameSpec(const std::string& configName,
                      const std::string& dcmOperatingMode,
                      const std::string& detectorId,
                      const std::string& subsystemName,
                      const std::string& configType);

    SubsystemNameSpec(const std::string& configName,
                      const std::string& dcmOperatingMode,
                      const std::string& detectorId,
                      const std::string& subsystemName,
                      const std::string& configType,
                      const std::string& timestamp);

    // *****************
    // *** Accessors ***
    // *****************

    std::string getConfigName() const {return _configName;}
    std::string getDCMOpMode() const {return _dcmOperatingMode;}
    std::string getDetectorId() const {return _detectorId;}
    std::string getDetectorName() const {return _detectorId;}

    std::string getSubsystemName() const {return _subsystemName;}
    std::string getConfigType() const {return _configType;}

    bool hasTimestamp() const {return _hasTimestamp;}
    std::string getTimestamp() const {return _timestamp;}

    bool isSlowControlsConfiguration() const {return _isSlowControlsConfig;}
    bool isDAQConfiguration() const {return (! _isSlowControlsConfig);}

    int getUsageCount() const {return _usageCount;}

    std::string toString(int overallIndent = 0) const;

    // *****************
    // *** Modifiers ***
    // *****************

    void isSlowControlsConfiguration(bool setting) {
        _isSlowControlsConfig = setting;
    }
    void isDAQConfiguration(bool setting) {
        _isSlowControlsConfig = (! setting);
    }
    void setUsageCount(int value) {_usageCount = value;}
    void setSortIndex(int value) {_sortIndex = value;}

    // *****************
    // *** Operators ***
    // *****************

    bool operator<(const SubsystemNameSpec& other) const;

 private:

    void _addSpaces(std::string& workString, int count) const;

    std::string _configName;
    std::string _dcmOperatingMode;
    std::string _detectorId;

    std::string _subsystemName;
    std::string _configType;

    bool _hasTimestamp;
    std::string _timestamp;

    bool _isSlowControlsConfig;

    int _usageCount;
    int _sortIndex;
};

};  // end of namespace daqconfig
};  // end of namespace dbutils

#endif
