#include "DAQDataFormats/RawConfigurationTailV0.h"
#include "DAQDataFormats/BitFields.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawConfigurationTail
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE rawconftail
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{
//------------------------------------------------------------------------------
// Default constructor
RawConfigurationTail::RawConfigurationTail(const version_t version):
             RawDAQData(version)
            ,ZEROOUT_GET_AND_SET(Version)
            ,ZEROOUT_GET_AND_SET(Reserved1)
            ,ZEROOUT_GET_AND_SET(Reserved2)
            ,ZEROOUT_GET_AND_SET(MarkerLo1)
            ,ZEROOUT_GET_AND_SET(MarkerLo2)
            ,ZEROOUT_GET_AND_SET(CRC)
{}

RawConfigurationTail::RawConfigurationTail (const RawConfigurationTail& copyin):
             RawDAQData(copyin)
            ,DEEPCOPY_GET_AND_SET(Version)
            ,DEEPCOPY_GET_AND_SET(Reserved1)
            ,DEEPCOPY_GET_AND_SET(Reserved2)
            ,DEEPCOPY_GET_AND_SET(MarkerLo1)
            ,DEEPCOPY_GET_AND_SET(MarkerLo2)
            ,DEEPCOPY_GET_AND_SET(CRC)
{}
//------------------------------------------------------------------------------
// Destructor
RawConfigurationTail::~RawConfigurationTail() { }


//------------------------------------------------------------------------------
// Initialization of data members
void RawConfigurationTail::init(){

    EXECUTE_FUNCTION(setVersion)(_version);

    // Set the Marker
    EXECUTE_FUNCTION(setMarkerLo1)(daqdataformats::rawconftail::Marker_value[0]);
    EXECUTE_FUNCTION(setMarkerLo2)(daqdataformats::rawconftail::Marker_value[1]);
}

//------------------------------------------------------------------------------
// Printing the RawConfigurationTail structure
void RawConfigurationTail::print(std::ostream& os) const{
    const uint32_t size_of_data = EXECUTE_FUNCTION(sizeofdata)();

    os<<boost::format("\n---RawConfigurationTail---------------------------------------------size = %u words\n")
            % size_of_data;

    os<<boost::format(" MarkerLo1=%u") % EXECUTE_FUNCTION(getMarkerLo1)();
    os<<boost::format(" MarkerLo2=%u") % EXECUTE_FUNCTION(getMarkerLo2)();

    os<<"\nRawConfigurationTail Data RAW 32bit WORDs (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < size_of_data; ++i){
        printWord(i, false, os);
        os<<" ";
    }

    os<<"\nRawConfigurationTail Data RAW 32bit Bits (converted, if required, to native endian format):";
    for(uint32_t i = 0; i < size_of_data; ++i){
            os<<"\n    ";
            printWord(i, true, os);
    }

}

}} // end of namespaces