#include "DAQDataFormats/RawMicroBlockHeader.h"
#include "DAQDataFormats/DataFormatException.h"

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawMicroBlockHeader
#define  LAST_CLASS_VERSION           1
#define  CURRENT_FIELDS_SUB_NAMESPACE microblock
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{

//------------------------------------------------------------
// Default constructor
RawMicroBlockHeader::RawMicroBlockHeader(const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__)
    /// Apply the default constructor
    defaultConstructorFixedSize();
}


//------------------------------------------------------------------------------
RawMicroBlockHeader::RawMicroBlockHeader(const RawMicroBlockHeader& copy_in):
    PREVIOUS_VERSION_CLASS_NAME(copy_in._version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__)
    copy(copy_in);
}

//------------------------------------------------------------------------------
RawMicroBlockHeader::RawMicroBlockHeader(const void* buffer, const version_t version):
    PREVIOUS_VERSION_CLASS_NAME(version)
{
    INITIALIZE_DEBUGGING_VARIABLES;
    PRINT_ON_DEBUG(__PRETTY_FUNCTION__)
    copy(buffer);
}

//------------------------------------------------------------
// Destructor
RawMicroBlockHeader::~RawMicroBlockHeader() { }

/// All binding routines
#define STANDARD_ROUTINES                                              \
            BIND_GET_AND_SET(DestBuffID          , version0, uint32_t);\
            BIND_GET_AND_SET(Partition           , version0, uint32_t);\
            BIND_GET_AND_SET(Version             , version0, uint32_t);\
            BIND_GET_AND_SET(V0                  , version0, uint32_t);\
            BIND_GET_AND_SET(V1                  , version0, uint32_t);\
            BIND_STADARD_FUNCTION(sizeofdata);                         \
            BIND_STADARD_FUNCTION(readData);                           \
            BIND_STADARD_FUNCTION(setBufferSource);                    

#include "DAQDataFormats/setFunctionPointers_template.inc"



}// end ofa namespace