#include "DAQDataFormats/RawTriggerHeaderV0.h"
#include <boost/format.hpp>

/// It is requred to
///   1. include DAQDataFormats/DataFormatFirst.inc
///   2. define CURRENT_CLASS
///   3.    define CURRENT_CLASS_VERSION for version of the class
///      OR define LAST_CLASS_VERSION    for the main structure of the class
///   4. (optional) define CURRENT_FIELDS_SUB_NAMESPACE
///   5. include DAQDataFormats/DataFormatLast.inc
/// before the declaration of the DataFormat version class in the header.
#include "DAQDataFormats/DataFormatFirst.inc"
#define  CURRENT_CLASS                RawTriggerHeader
#define  CURRENT_CLASS_VERSION        0
#define  CURRENT_FIELDS_SUB_NAMESPACE triggerheader
#include "DAQDataFormats/DataFormatLast.inc"

namespace daqdataformats{
namespace VERSION_NAMESPACE{

//------------------------Constructors------------------------------------------
RawTriggerHeader::RawTriggerHeader(const version_t version):
             PREVIOUS_VERSION_CLASS_NAME(version)
            ,ZEROOUT_GET_AND_SET(Delimiter              )
            ,ZEROOUT_GET_AND_SET(Version                )
            ,ZEROOUT_GET_AND_SET(HWFlag                 )
            ,ZEROOUT_GET_AND_SET(Source                 )
            ,ZEROOUT_GET_AND_SET(SourceID               )
            ,ZEROOUT_GET_AND_SET(SourceSubID            )
            ,ZEROOUT_GET_AND_SET(TriggerNumberLow       )
            ,ZEROOUT_GET_AND_SET(TriggerNumberHigh      )
            ,ZEROOUT_GET_AND_SET(MasterTriggerNumberLow )
            ,ZEROOUT_GET_AND_SET(MasterTriggerNumberHigh)
            ,ZEROOUT_GET_AND_SET(MasterTriggerNumber)
            ,ZEROOUT_GET_AND_SET(TriggerNumber)
{}

//------------------------------------------------------------------------------
RawTriggerHeader::RawTriggerHeader(const RawTriggerHeader& copyin):
             PREVIOUS_VERSION_CLASS_NAME(copyin)
            ,DEEPCOPY_GET_AND_SET(Delimiter              )
            ,DEEPCOPY_GET_AND_SET(Version                )
            ,DEEPCOPY_GET_AND_SET(HWFlag                 )
            ,DEEPCOPY_GET_AND_SET(Source                 )
            ,DEEPCOPY_GET_AND_SET(SourceID               )
            ,DEEPCOPY_GET_AND_SET(SourceSubID            )
            ,DEEPCOPY_GET_AND_SET(TriggerNumberLow       )
            ,DEEPCOPY_GET_AND_SET(TriggerNumberHigh      )
            ,DEEPCOPY_GET_AND_SET(MasterTriggerNumberLow )
            ,DEEPCOPY_GET_AND_SET(MasterTriggerNumberHigh)
            ,DEEPCOPY_GET_AND_SET(MasterTriggerNumber)
            ,DEEPCOPY_GET_AND_SET(TriggerNumber)
{}
 
//------------------------------------------------------------------------------
RawTriggerHeader::~RawTriggerHeader(){ }
//------------------------------------------------------------------------------
// General Initialization 
void RawTriggerHeader::init(){
    EXECUTE_FUNCTION(setVersion)(_version);
    EXECUTE_FUNCTION(setDelimiter)(daqdataformats::triggerheader::TriggerHeaderDelimiter);
}

//------------------------------------------------------------------------------
// get Method for a full (64bit) wide trigger number (40bits in use)
uint64_t RawTriggerHeader::getTriggerNumber() const { 
    return getUint64_t(EXECUTE_FUNCTION(getTriggerNumberLow)(), EXECUTE_FUNCTION(getTriggerNumberHigh)());
}

//------------------------------------------------------------------------------
// get Method for a full (64bit) wide trigger number (40bits in use)
uint64_t RawTriggerHeader::getMasterTriggerNumber() const {
    return getUint64_t(EXECUTE_FUNCTION(getMasterTriggerNumberLow)(), EXECUTE_FUNCTION(getMasterTriggerNumberHigh)());
}

//------------------------------------------------------------------------------
// set Method for Trigger Number High
bool RawTriggerHeader::setTriggerNumber(const uint64_t trig)      { 
      uint32_t low, high;
      getLoAndHi(trig, low, high);

      EXECUTE_FUNCTION(setTriggerNumberHigh)(high);
      RETURN_FUNCTION(setTriggerNumberLow)(low);
}

//------------------------------------------------------------------------------
// set Method for Trigger Number High
bool RawTriggerHeader::setMasterTriggerNumber(const uint64_t trig){ 
      uint32_t low, high;
      getLoAndHi(trig, low, high);

      EXECUTE_FUNCTION(setMasterTriggerNumberHigh)(high);
      RETURN_FUNCTION(setMasterTriggerNumberLow)(low);
}

//------------------------------------------------------------------------------
void RawTriggerHeader::print(std::ostream& os) const {

    os<<"++++ Trigger Header ++++\n";
    os<<boost::format("Trigger Delimiter:\t0x%04x\n")%EXECUTE_FUNCTION(getDelimiter)();
    os<<boost::format("Trigger Version:\t%u\n")%EXECUTE_FUNCTION(getVersion)();
    os<<boost::format("Hardware Flag:\t\t%u\n")%EXECUTE_FUNCTION(getHWFlag)();
    os<<boost::format("Trigger Source:\t\t%u\n")%EXECUTE_FUNCTION(getSource)();
    os<<boost::format("Trigger Source ID:\t%u\n")%EXECUTE_FUNCTION(getSourceID)();
    os<<boost::format("Trigger Source Sub ID:\t%u\n")%EXECUTE_FUNCTION(getSourceSubID)();
    os<<boost::format("Trigger Number Low:\t%u\n")%EXECUTE_FUNCTION(getTriggerNumberLow)();
    os<<boost::format("Trigger Number High:\t%u\n")%EXECUTE_FUNCTION(getTriggerNumberHigh)();
    os<<boost::format("Master Trig Num Low:\t%u\n")%EXECUTE_FUNCTION(getMasterTriggerNumberLow)();
    os<<boost::format("Master Trig Num High:\t%u\n")%EXECUTE_FUNCTION(getMasterTriggerNumberHigh)();
    PRINT_ON_DEBUG("Buffer is set to "<<_Buffer)
}

}}// end of namespaces